
'use strict';

const GLib = imports.gi.GLib;
const Gio = imports.gi.Gio;
const Main = imports.ui.main;

const ByteArray = imports.byteArray;

const docker_commands_to_labels = {
    start: 'Start',
	"start i": 'Start interactive',
    restart: 'Restart',
    stop: 'Stop',
    pause: 'Pause',
    unpause: 'Unpause',
    rm: 'Remove',
    exec: 'Exec'
};

/**
 * Check if docker is installed
 * @return {Boolean} whether docker is installed or not
 */
function is_docker_installed() {
	return !!GLib.find_program_in_path('docker');
}

/**
 * Return containers name and status.
 * @return {Array.<{name: String, status: String}>} every object represents a container
 */
function get_containers() {
	let command = `docker container list --all --format "{{.Names}};{{.State}}"`;

	let proc = Gio.Subprocess.new(
		// The program and command options are passed as a list of arguments
		command.split(" "),

		// The flags control what I/O pipes are opened and how they are directed
		Gio.SubprocessFlags.STDOUT_PIPE | Gio.SubprocessFlags.STDERR_PIPE
	);

	let out = "";
	let loop = GLib.MainLoop.new(null, false);
	proc.communicate_utf8_async(null, null, (proc, res) => {
		try {
			let [, stdout, stderr] = proc.communicate_utf8_finish(res);

			if (!proc.get_successful()) {
				throw new Error(stderr);
			}

			out = stdout;
		} catch (e) {
			Main.notifyError('Docker ERROR', `${e}`);
		} finally {
			loop.quit();
		}
	});
	loop.run();

	if (!out.length) {
		return [];
	}

	// Found containers
	return out
			.replaceAll("\"","")
			.split("\n")
			.filter(element => element)
			.map(str => str.split(";"))
			.map(s => {
				return {
					name:s[0],
					status:s[1]
				};
			});

}

/**
 * Run docker command.
 * @param {String} command - docker command
 * @param {String} name - name of container
 */
function run_command(command, name, cancellable = null) {
	let c = "";
	switch (command){
		case "exec":
			c = `gnome-terminal -- sh -c 'docker exec -it ${name} bash' `

			GLib.spawn_command_line_async(c);
		return;

		case "start i":
			c = `gnome-terminal -- sh -c 'docker start -i ${name}; exec $SHELL' `

			GLib.spawn_command_line_async(c);
		return;

		default:
			c = `docker ${command} ${name}`
	}

	let proc = Gio.Subprocess.new(
		c.split(" "),
		Gio.SubprocessFlags.STDOUT_PIPE | Gio.SubprocessFlags.STDERR_PIPE
	);


	proc.communicate_utf8_async(null, null, (proc, res) => {
		try {
			let [, , stderr] = proc.communicate_utf8_finish(res);

			if (!proc.get_successful()) {
				throw new Error(stderr);
			}

			Main.notify(`${docker_commands_to_labels[command]} container: ${name}`);
		} catch (e) {
			Main.notifyError('Docker ERROR', `${e}`);
		}
	});
}
